@@ -2,6 +2,7 @@ package configurer
 
 import (
 	"fmt"
+	"path"
 	"regexp"
 	"strconv"
 	"strings"
@@ -17,7 +18,7 @@ type PathFuncs interface {
 	K0sBinaryPath() string
 	K0sConfigPath() string
 	K0sJoinTokenPath() string
-	KubeconfigPath(h os.Host) string
+	KubeconfigPath(os.Host, string) string
 	DataDirDefaultPath() string
 }
 
@@ -137,12 +138,15 @@ func (l Linux) MoveFile(h os.Host, src, dst string) error {
 }
 
 // KubeconfigPath returns the path to a kubeconfig on the host
-func (l Linux) KubeconfigPath(h os.Host) string {
+func (l Linux) KubeconfigPath(h os.Host, dataDir string) string {
 	linux := &os.Linux{}
-	if linux.FileExist(h, "/var/lib/k0s/pki/admin.conf") {
-		return "/var/lib/k0s/pki/admin.conf"
+
+	// if admin.conf exists, use that
+	adminConfPath := path.Join(dataDir, "pki/admin.conf")
+	if linux.FileExist(h, adminConfPath) {
+		return adminConfPath
 	}
-	return "/var/lib/k0s/kubelet.conf"
+	return path.Join(dataDir, "kubelet.conf")
 }
 
 // DataDirPath returns the location of k0s data dir
@@ -151,8 +155,8 @@ func (l Linux) DataDirDefaultPath() string {
 }
 
 // KubectlCmdf returns a command line in sprintf manner for running kubectl on the host using the kubeconfig from KubeconfigPath
-func (l Linux) KubectlCmdf(h os.Host, s string, args ...interface{}) string {
-	return fmt.Sprintf(`env "KUBECONFIG=%s" %s`, l.PathFuncs.KubeconfigPath(h), l.K0sCmdf(`kubectl %s`, fmt.Sprintf(s, args...)))
+func (l Linux) KubectlCmdf(h os.Host, dataDir, s string, args ...interface{}) string {
+	return fmt.Sprintf(`env "KUBECONFIG=%s" %s`, l.PathFuncs.KubeconfigPath(h, dataDir), l.K0sCmdf(`kubectl %s`, fmt.Sprintf(s, args...)))
 }
 
 // HTTPStatus makes a HTTP GET request to the url and returns the status code or an error
