@@ -1,6 +1,8 @@
 package phase
 
 import (
+	"fmt"
+
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
 	log "github.com/sirupsen/logrus"
@@ -53,6 +55,11 @@ func (p *InitializeK0s) Run() error {
 		p.SetProp("dynamic-config", true)
 	}
 
+	if Force {
+		log.Warnf("%s: --force given, using k0s install with --force", h)
+		h.InstallFlags.AddOrReplace("--force=true")
+	}
+
 	log.Infof("%s: installing k0s controller", h)
 	cmd, err := h.K0sInstallCommand()
 	if err != nil {
@@ -87,6 +94,12 @@ func (p *InitializeK0s) Run() error {
 		return err
 	}
 
+	if p.Config.Spec.K0s.DynamicConfig {
+		if err := h.WaitK0sDynamicConfigReady(); err != nil {
+			return fmt.Errorf("dynamic config reconciliation failed: %w", err)
+		}
+	}
+
 	h.Metadata.K0sRunningVersion = p.Config.Spec.K0s.Version
 	h.Metadata.K0sBinaryVersion = p.Config.Spec.K0s.Version
 
