@@ -27,7 +27,7 @@ func (p *InstallControllers) Prepare(config *v1beta1.Cluster) error {
 	var controllers cluster.Hosts = p.Config.Spec.Hosts.Controllers()
 	p.leader = p.Config.Spec.K0sLeader()
 	p.hosts = controllers.Filter(func(h *cluster.Host) bool {
-		return !h.Reset && (h != p.leader && h.Metadata.K0sRunningVersion == "")
+		return !h.Reset && !h.Metadata.NeedsUpgrade && (h != p.leader && h.Metadata.K0sRunningVersion == "")
 	})
 
 	return nil
@@ -67,7 +67,7 @@ func (p *InstallControllers) Run() error {
 		}
 		log.Debugf("%s: join token ID: %s", p.leader, tokenID)
 		defer func() {
-			if err := p.leader.Exec(p.leader.Configurer.K0sCmdf("token invalidate --data-dir=%s %s", h.DataDir, tokenID), exec.Sudo(p.leader), exec.RedactString(token)); err != nil {
+			if err := p.leader.Exec(p.leader.Configurer.K0sCmdf("token invalidate --data-dir=%s %s", h.K0sDataDir(), tokenID), exec.Sudo(p.leader), exec.RedactString(token)); err != nil {
 				log.Warnf("%s: failed to invalidate the controller join token", p.leader)
 			}
 		}()
@@ -87,6 +87,11 @@ func (p *InstallControllers) Run() error {
 			h.InstallFlags.AddOrReplace("--enable-dynamic-config")
 		}
 
+		if Force {
+			log.Warnf("%s: --force given, using k0s install with --force", h)
+			h.InstallFlags.AddOrReplace("--force=true")
+		}
+
 		log.Infof("%s: installing k0s controller", h)
 		cmd, err := h.K0sInstallCommand()
 		if err != nil {
